/**
 * @module package/quiqqer/crm/bin/backend/controls/TaskList
 * @author www.pcsg.de (Henning Leutz)
 */
define('package/quiqqer/crm/bin/backend/controls/CreateTaskWindow', [

    'qui/QUI',
    'qui/controls/windows/Confirm',
    'utils/Controls',
    'Ajax',
    'Locale',
    'Mustache',

    'text!package/quiqqer/crm/bin/backend/controls/CreateTaskWindow.html',
    'css!package/quiqqer/crm/bin/backend/controls/CreateTaskWindow.css'

], function (QUI, QUIConfirm, ControlUtils, QUIAjax, QUILocale, Mustache, template) {
    "use strict";

    var lg = 'quiqqer/crm';

    return new Class({

        Extends: QUIConfirm,
        Type   : 'package/quiqqer/crm/bin/backend/controls/CreateTaskWindow',

        Binds: [
            '$onOpen'
        ],

        options: {
            icon     : 'fa fa-plus',
            maxHeight: 600,
            maxWidth : 800,

            customerId        : false,
            customerChangeable: true
        },

        initialize: function (options) {
            this.parent(options);

            this.setAttributes({
                title    : QUILocale.get(lg, 'window.create.title'),
                ok_button: {
                    text     : QUILocale.get('quiqqer/quiqqer', 'add'),
                    textimage: 'fa fa-check'
                }
            });

            this.addEvents({
                onOpen: this.$onOpen
            });
        },

        /**
         * event: on open
         */
        $onOpen: function () {
            var self = this;

            this.Loader.show();
            this.getContent().addClass('quiqqer-crm-create-task-window');
            this.getContent().set('html', Mustache.render(template, {}));

            if (this.getAttribute('customerId')) {
                this.getContent().getElement('[name="customer"]').value = this.getAttribute('customerId');
            }

            QUI.parse(this.getContent()).then(function () {
                return ControlUtils.parse(self.getContent());
            }).then(function () {
                if (self.getAttribute('customerChangeable') === false) {
                    var Customer = QUI.Controls.getById(
                        self.getContent().getElement('[name="customer"]').get('data-quiid')
                    );

                    Customer.disable();
                }

                self.Loader.hide();
            });
        },

        /**
         * Submit the window
         *
         * @method qui/controls/windows/Confirm#submit
         */
        submit: function () {
            var self       = this;
            var Form       = this.getElm().getElement('form'),
                CustomerId = Form.getElement('[name="customer"]');

            if (CustomerId.value === '') {
                QUI.getMessageHandler().then(function (MH) {
                    MH.addError(
                        QUILocale.get(lg, 'exception.missing.customer'),
                        CustomerId.getParent()
                    );
                });

                return;
            }

            this.Loader.show();

            QUIAjax.post('package_quiqqer_crm_ajax_backend_tasks_create', function () {
                self.fireEvent('submit', [this]);

                if (self.getAttribute('autoclose')) {
                    self.close();
                }
            }, {
                'package' : 'quiqqer',
                attributes: JSON.encode({
                    title     : Form.getElement('[name="title"]').value,
                    customerId: Form.getElement('[name="customer"]').value,
                    from      : Form.getElement('[name="from"]').value,
                    to        : Form.getElement('[name="to"]').value,
                    text      : Form.getElement('[name="text"]').value
                }),
                onError   : function () {
                    self.Loader.hide();
                }
            });
        }
    });
});
