/**
 * @module package/quiqqer/crm/bin/backend/controls/EditTaskWindow
 * @author www.pcsg.de (Henning Leutz)
 */
define('package/quiqqer/crm/bin/backend/controls/EditTaskWindow', [

    'qui/QUI',
    'qui/controls/windows/Confirm',
    'utils/Controls',
    'package/quiqqer/erp/bin/backend/controls/Comments',
    'Ajax',
    'Locale',
    'Mustache',

    'text!package/quiqqer/crm/bin/backend/controls/CreateTaskWindow.html',
    'css!package/quiqqer/crm/bin/backend/controls/CreateTaskWindow.css'

], function (QUI, QUIConfirm, ControlUtils, Comments, QUIAjax, QUILocale, Mustache, template) {
    "use strict";

    var lg = 'quiqqer/crm';

    return new Class({

        Extends: QUIConfirm,
        Type   : 'package/quiqqer/crm/bin/backend/controls/EditTaskWindow',

        Binds: [
            'openAddCommentWindow',
            '$onOpen'
        ],

        options: {
            icon     : 'fa fa-edit',
            maxHeight: Math.round(document.body.getSize().y * 0.9),
            maxWidth : 800,

            taskId            : false,
            customerChangeable: false
        },

        initialize: function (options) {
            this.parent(options);

            this.$Comments = null;

            this.setAttributes({
                title    : QUILocale.get(lg, 'window.edit.title'),
                ok_button: {
                    text     : QUILocale.get('quiqqer/quiqqer', 'edit'),
                    textimage: 'fa fa-check'
                }
            });

            this.addEvents({
                onOpen: this.$onOpen
            });
        },

        /**
         * event: on open
         */
        $onOpen: function () {
            var self = this;

            this.getContent().addClass('quiqqer-crm-edit-task-window');
            this.getContent().set('html', Mustache.render(template, {}));

            this.Loader.show();
            var Form = self.getElm().getElement('form');

            this.refresh(true).then(function () {
                Form.getElement('[name="comments"]').setStyle('display', null);
                Form.getElement('[name="add-comment"]').addEvent('click', self.openAddCommentWindow);

                return QUI.parse(self.getContent());
            }).then(function () {
                return ControlUtils.parse(self.getContent());
            }).then(function () {
                if (self.getAttribute('customerChangeable') === false) {
                    var Customer = QUI.Controls.getById(
                        self.getContent().getElement('[name="customer"]').get('data-quiid')
                    );

                    Customer.disable();
                }

                self.Loader.hide();
            });
        },

        /**
         * Refresh the display
         */
        refresh: function (noLoader) {
            var self = this;

            if (typeof noLoader !== 'undefined' && noLoader) {
                this.Loader.show();
            }

            return new Promise(function (resolve) {
                QUIAjax.get('package_quiqqer_crm_ajax_backend_tasks_get', function (task) {
                    var Form        = self.getElm().getElement('form');
                    var CommentCell = Form.getElement('[name="comments"] tbody td');
                    var status      = parseInt(task.status);

                    if (status === 0) {
                        status = 1;
                    }

                    Form.getElement('[name="title"]').value    = task.title;
                    Form.getElement('[name="customer"]').value = task.userId;
                    Form.getElement('[name="from"]').value     = task.from;
                    Form.getElement('[name="to"]').value       = task.to;
                    Form.getElement('[name="text"]').value     = task.text;
                    Form.getElement('[name="status"]').value   = status;
                    console.log(task);
                    if (!self.$Comments) {
                        self.$Comments = new Comments({
                            comments: task.comments
                        }).inject(CommentCell);
                    } else {
                        self.$Comments.unserialize(task.comments);
                    }

                    if (typeof noLoader !== 'undefined' && noLoader) {
                        self.Loader.show();
                    }

                    resolve();
                }, {
                    'package': 'quiqqer/crm',
                    taskId   : self.getAttribute('taskId')
                });
            });
        },

        /**
         * Submit the window
         *
         * @method qui/controls/windows/Confirm#submit
         */
        submit: function () {
            var self = this;
            var Form = this.getElm().getElement('form');

            this.Loader.show();

            QUIAjax.post('package_quiqqer_crm_ajax_backend_tasks_save', function () {
                self.fireEvent('submit', [this]);

                if (self.getAttribute('autoclose')) {
                    self.close();
                }
            }, {
                'package' : 'quiqqer/crm',
                taskId    : this.getAttribute('taskId'),
                attributes: JSON.encode({
                    title     : Form.getElement('[name="title"]').value,
                    customerId: Form.getElement('[name="customer"]').value,
                    from      : Form.getElement('[name="from"]').value,
                    to        : Form.getElement('[name="to"]').value,
                    text      : Form.getElement('[name="text"]').value,
                    status    : parseInt(Form.getElement('[name="status"]').value)
                })
            });
        },

        /**
         * open the add comment window
         */
        openAddCommentWindow: function () {
            var self = this;

            require(['qui/controls/windows/Confirm'], function (QUIConfirm) {
                new QUIConfirm({
                    icon     : 'fa fa-comment',
                    title    : QUILocale.get(lg, 'window.add.comment.title'),
                    maxHeight: self.getElm().getSize().y,
                    maxWidth : self.getElm().getSize().x,
                    autoclose: true,
                    events   : {
                        onOpen: function (Win) {
                            var Content = Win.getContent();

                            Content.addClass('quiqqer-crm-edit-task-window-add-comment');
                            Content.set(
                                'html',

                                QUILocale.get(lg, 'window.add.comment.text') +
                                '<textarea name="add-comment-field"></textarea>'
                            );
                        },

                        onSubmit: function (Win) {
                            var Content  = Win.getContent();
                            var Textarea = Content.getElement('textarea');

                            if (Textarea.value === '') {
                                return;
                            }

                            Win.Loader.show();

                            QUIAjax.post('package_quiqqer_crm_ajax_backend_tasks_addComment', function () {
                                Win.close();
                                self.fireEvent('commentCreated', [self]);
                                self.refresh();
                            }, {
                                'package': 'quiqqer/crm',
                                taskId   : self.getAttribute('taskId'),
                                message  : Textarea.value,
                                onError  : function () {
                                    Win.Loader.hide();
                                }
                            });
                        }
                    }
                }).open();
            });
        }
    });
});
