<?php

namespace QUI\ERP\CRM;

use QUI;

/**
 * Class Handler
 *
 * @package QUI\ERP\CRM
 */
class TaskHandler
{
    public const TASK_TYPE_TASK = 'task';

    /**
     * @return string
     */
    public static function table()
    {
        return QUI::getDBTableName('customer_tasks');
    }

    /**
     * @param $taskId
     * @return Task
     *
     * @throws TaskException
     */
    public static function getTask($taskId)
    {
        return new Task($taskId);
    }

    /**
     * Return the data of a task
     *
     * @param $taskId
     * @return array
     *
     * @throws TaskException
     */
    public static function getTaskData($taskId)
    {
        try {
            $result = QUI::getDataBase()->fetch([
                'from'  => self::table(),
                'where' => [
                    'id' => $taskId
                ]
            ]);
        } catch (QUI\Exception $Exception) {
            throw new TaskException('Task not found', 404);
        }

        if (!isset($result[0])) {
            throw new TaskException('Task not found', 404);
        }

        return $result[0];
    }

    //region general

    /**
     *
     * @param array $query
     * @return Task[]
     */
    public static function getTasks($query = [])
    {
        $dbQuery = [
            'select' => 'id',
            'from'   => self::table()
        ];

        if (isset($query['where'])) {
            $dbQuery['where'] = $query['where'];
        }

        if (isset($query['order'])) {
            $dbQuery['order'] = $query['order'];
        }

        if (isset($query['limit'])) {
            $dbQuery['limit'] = $query['limit'];
        }

        try {
            $result = QUI::getDataBase()->fetch($dbQuery);
        } catch (QUI\Exception $Exception) {
            return [];
        }

        return self::parseDbIdResultToTasks($result);
    }

    /**
     * @param array $query
     */
    public static function countTasks($query = [])
    {
        try {
            $result = QUI::getDataBase()->fetch([
                'count' => [
                    'select' => 'id',
                    'as'     => 'count'
                ],
                'from'  => self::table()
            ]);
        } catch (QUI\Exception $Exception) {
            return 0;
        }

        return (int)$result[0]['count'];
    }

    //endregion

    //region by customer

    /**
     * @param integer $userId - ID of the customer
     * @param array $query - optional, order & limit
     *
     * @return Task[]
     */
    public static function getTasksByCustomer($userId, $query = [])
    {
        $dbQuery = [
            'select' => 'id',
            'from'   => self::table(),
            'where'  => [
                'userId' => (int)$userId
            ]
        ];

        if (isset($query['order'])) {
            $dbQuery['order'] = $query['order'];
        }

        if (isset($query['limit'])) {
            $dbQuery['limit'] = $query['limit'];
        }

        try {
            $result = QUI::getDataBase()->fetch($dbQuery);
        } catch (QUI\Exception $Exception) {
            return [];
        }

        return self::parseDbIdResultToTasks($result);
    }

    /**
     * @param $userId
     * @param array $query
     */
    public static function countTasksByCustomer($userId, $query = [])
    {
        try {
            $result = QUI::getDataBase()->fetch([
                'count' => [
                    'select' => 'id',
                    'as'     => 'count'
                ],
                'from'  => self::table(),
                'where' => [
                    'userId' => (int)$userId
                ]
            ]);
        } catch (QUI\Exception $Exception) {
            return 0;
        }

        return (int)$result[0]['count'];
    }

    //endregion

    //region by editor

    /**
     * @param integer $userId - ID of the editor
     * @return Task[]
     */
    public static function getTasksByEditor($userId)
    {
        try {
            $result = QUI::getDataBase()->fetch([
                'select' => 'id',
                'from'   => self::table(),
                'where'  => [
                    'editorId' => (int)$userId
                ]
            ]);
        } catch (QUI\Exception $Exception) {
            return [];
        }

        return self::parseDbIdResultToTasks($result);
    }

    //endregion

    /**
     * @param $result
     * @return Task[]
     */
    protected static function parseDbIdResultToTasks($result)
    {
        $tasks = [];

        foreach ($result as $entry) {
            try {
                $tasks[] = self::getTask($entry['id']);
            } catch (TaskException $Exception) {
                QUI\System\Log::addDebug($Exception->getMessage());
            }
        }

        return $tasks;
    }
}
