<?php

/**
 * This file contains QUI\ERP\CRM\TaskStatus\Factory
 */

namespace QUI\ERP\CRM\TaskStatus;

use QUI;

/**
 * Class Factory
 * - For task status creation
 *
 * @package QUI\ERP\CRM\TaskStatus
 */
class Factory extends QUI\Utils\Singleton
{
    /**
     * Create a new task status
     *
     * @param string|integer $id - task ID
     * @param string $color - color of the status
     * @param array $title - title
     *
     * @throws Exception
     * @throws QUI\Exception
     * @todo permissions
     */
    public function createTaskStatus($id, $color, array $title)
    {
        $list = Handler::getInstance()->getList();
        $id   = (int)$id;
        $data = [];

        if (isset($list[$id])) {
            throw new Exception([
                'quiqqer/crm',
                'exception.processStatus.exists'
            ]);
        }

        // config
        $Package = QUI::getPackage('quiqqer/crm');
        $Config  = $Package->getConfig();

        $Config->setValue('task_status', $id, $color);
        $Config->save();

        // translations
        if (\is_array($title)) {
            $languages = QUI::availableLanguages();

            foreach ($languages as $language) {
                if (isset($title[$language])) {
                    $data[$language] = $title[$language];
                }
            }
        }

        // TaskSatus title
        $data['package']  = 'quiqqer/crm';
        $data['datatype'] = 'php,js';
        $data['html']     = 1;

        QUI\Translator::addUserVar(
            'quiqqer/crm',
            'task.status.'.$id,
            $data
        );

        QUI\Translator::publish('quiqqer/crm');

        Handler::getInstance()->refreshList();
    }

    /**
     * Return a next ID to create a new Task Status
     *
     * @return int
     */
    public function getNextId()
    {
        $list = Handler::getInstance()->getList();

        if (!\count($list)) {
            return 1;
        }

        $max = \max(\array_keys($list));

        return $max + 1;
    }
}
