<?php

/**
 * This file contains QUI\ERP\CRM\TaskStatus\Handler
 */

namespace QUI\ERP\CRM\TaskStatus;

use QUI;

/**
 * Class Handler
 * - Task status management
 * - Returns task status objects
 * - Returns task status lists
 *
 * @package QUI\ERP\CRM\TaskStatus\Factory
 */
class Handler extends QUI\Utils\Singleton
{
    /**
     * @var array
     */
    protected $list = null;

    /**
     * Return all task status entries from the config
     *
     * @return array
     */
    public function getList()
    {
        if ($this->list !== null) {
            return $this->list;
        }

        try {
            $Package = QUI::getPackage('quiqqer/crm');
            $Config  = $Package->getConfig();
            $result  = $Config->getSection('task_status');
        } catch (QUI\Exception $Exception) {
            QUI\System\Log::writeException($Exception);

            return [];
        }

        if (!$result || !\is_array($result)) {
            $this->list = [];

            return $this->list;
        }

        $this->list = $result;

        return $result;
    }

    /**
     * Refresh the internal list
     *
     * @return array
     */
    public function refreshList()
    {
        $this->list = null;

        return $this->getList();
    }

    /**
     * Return the complete task status objects
     *
     * @return Status[]
     */
    public function getTaskStatusList()
    {
        $list   = $this->getList();
        $result = [];

        foreach ($list as $entry => $color) {
            try {
                $result[] = $this->getTaskStatus($entry);
            } catch (Exception $Exception) {
            }
        }

        return $result;
    }

    /**
     * Return a task status
     *
     * @param $id
     * @return Status|StatusUnknown
     *
     * @throws Exception
     */
    public function getTaskStatus($id)
    {
        if ($id === 0) {
            return new StatusUnknown();
        }

        return new Status($id);
    }

    /**
     * Delete / Remove a task status
     *
     * @param string|int $id
     *
     * @throws Exception
     * @throws QUI\Exception
     *
     * @todo permissions
     */
    public function deleteTaskStatus($id)
    {
        $Status = $this->getTaskStatus($id);

        // remove translation
        QUI\Translator::delete(
            'quiqqer/crm',
            'task.status.'.$Status->getId()
        );

        QUI\Translator::publish('quiqqer/crm');

        // update config
        $Package = QUI::getPackage('quiqqer/crm');
        $Config  = $Package->getConfig();

        $Config->del('task_status', $Status->getId());
        $Config->save();
    }

    /**
     * Update a task status
     *
     * @param int|string $id
     * @param int|string $color
     * @param array $title
     *
     * @throws QUI\Exception
     *
     * @todo permissions
     */
    public function updateTaskStatus($id, $color, array $title)
    {
        $Status = $this->getTaskStatus($id);

        // update translation
        $languages = QUI::availableLanguages();

        $data = [
            'package'  => 'quiqqer/crm',
            'datatype' => 'php,js',
            'html'     => 1
        ];

        foreach ($languages as $language) {
            if (isset($title[$language])) {
                $data[$language]         = $title[$language];
                $data[$language.'_edit'] = $title[$language];
            }
        }

        QUI\Translator::edit(
            'quiqqer/crm',
            'task.status.'.$Status->getId(),
            'quiqqer/crm',
            $data
        );

        QUI\Translator::publish('quiqqer/crm');

        // update config
        $Package = QUI::getPackage('quiqqer/crm');
        $Config  = $Package->getConfig();

        $Config->setValue('task_status', $Status->getId(), $color);
        $Config->save();
    }
}
